# Copyright 2020 Spotify AB
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Common exceptions for use within the Klio ecosystem."""


class KlioConfigTemplatingException(Exception):
    """Exception for missing keys when overriding Klio config parameters

    Args:
        key (str): missing configuration key
    """

    def __init__(self, key):
        message = "{} missing in key template overrides.".format(key)
        super().__init__(message)
